--author: igor29381

Workers = {}

function Workers.prerequisitesPresent(specializations)
    return true;
end

Workers.curModDir = g_currentModDirectory;

function Workers:load(savegame)
    if hasXMLProperty(self.xmlFile, "vehicle.worker") then
		local i3dFilename = Workers.curModDir..getXMLString(self.xmlFile, "vehicle.worker#i3dFilename");
		if fileExists(i3dFilename) then
			local c = 0;
			self.skeletons = {};
			self.graphs = {};
			self.animCharSets = {};
			while true do
				local baseString = string.format("vehicle.worker.workerPoint(%d)#",c);
				local index = getXMLString(self.xmlFile,baseString.."index");
				if index == nil then break; end;
				index = Utils.indexToObject(self.components, index);
				local WorkerRoot = Utils.loadSharedI3DFile(i3dFilename);
				local skeletonId = getChildAt(WorkerRoot, 0);
				local graphId = getChildAt(WorkerRoot, 1);
				link(index, skeletonId);
				link(getRootNode(), graphId);
				setVisibility(graphId, false);
				delete(WorkerRoot);
				local animCharSet = getAnimCharacterSet(skeletonId);
				if animCharSet ~= 0 then
					local clip = getAnimClipIndex(animCharSet, "Worker");
					if clip > -1 then
						assignAnimTrackClip(animCharSet, 1, clip);
						setAnimTrackLoopState(animCharSet, 1, false);
						setAnimTrackSpeedScale(animCharSet, 1, 1);
						self.clipDuration = getAnimClipDuration(animCharSet, clip);
					end;
				end;
				table.insert(self.skeletons, skeletonId);
				table.insert(self.graphs, graphId);
				table.insert(self.animCharSets, animCharSet);
				if self.washableNodes ~= nil then
				 self.washableNodes[graphId] = graphId;
				end;
				self.isEnabled = true;
				c = c + 1;
			end;
		end;
	end;
end;

function Workers:delete()
    if self.skeletons and self.skeletons[1] then
        for i=1, #self.skeletons do
            delete(self.skeletons[i]);
            delete(self.graphs[i]);
        end;
    end;
end;

function Workers:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Workers:keyEvent(unicode, sym, modifier, isDown)
end;

function Workers:readUpdateStream(streamId, timestamp, connection)
end;

function Workers:writeUpdateStream(streamId, connection, dirtyMask)
end;

function Workers:update(dt)
    if self.isEnabled then
        local jointDescIndex = 0;
		if self.attacherVehicle then
			for i=1, #self.attacherVehicle.attachedImplements do
				local implement = self.attacherVehicle.attachedImplements[i];
				if implement.object == self then
					jointDescIndex = implement.jointDescIndex;
				end;
			end;
		end;
		if jointDescIndex > 0 then
			local jointDesc = self.attacherVehicle.attacherJoints[jointDescIndex];
			if jointDesc then
				local x,y,z = worldToLocal(self.rootNode, getWorldTranslation(getCamera()));
				local distance = Utils.vector3Length(x,y,z);
				if self:getFillLevel(FillUtil.FILLTYPE_SEEDS) > 0 then
					if not getVisibility(self.graphs[1]) then
						for _,graph in pairs(self.graphs) do
							setVisibility(graph, true);
						end;
					end;
				else
					if getVisibility(self.graphs[1]) then
						for _,graph in pairs(self.graphs) do
							setVisibility(graph, false);
						end;
					end;
				end;
				if distance < 40 and jointDesc.moveDown and self.movingDirection > 0 and (not self.needsActivation or self:getIsTurnedOn()) then
					if not isAnimTrackEnabled(self.animCharSets[1], 1) then
						for _,animCharSet in pairs(self.animCharSets) do
							enableAnimTrack(animCharSet, 1);
						end;
					end;
					for _,animCharSet in pairs(self.animCharSets) do
						if getAnimTrackTime(animCharSet, 1) >= self.clipDuration then
							local speed = math.random(10, 15)/10;
							setAnimTrackSpeedScale(animCharSet, 1, speed);
							setAnimTrackTime(animCharSet, 1, 0, false);
						end;
					end;
				elseif isAnimTrackEnabled(self.animCharSets[1], 1) then
					for _,animCharSet in pairs(self.animCharSets) do
						if getAnimTrackTime(animCharSet, 1) >= self.clipDuration then
							disableAnimTrack(animCharSet, 1);
							setAnimTrackTime(animCharSet, 1, 0, false);
						end;
					end;
				end;
			end;
		end;
    end;
end;

function Workers:updateTick(dt)
end;

function Workers:draw()
end;
